#
# Copyright (C) 2001, 2002 Caldera International, Inc.  All Rights Reserved.
#
# Library routines and global defines for Cyrus abstraction api.
#
# depends upon the globals set up by msglib.pm
#
# you need to be admin to use this API.
#
# the big thing this API brings other than the niceties of an abstraction
# is connection caching for updating or querying large number of users
# spread accross multiple machines.
#

# external routines are:
#
# cyrinit - initialize this module
# cyrdone - uninitialize this module (closes all cached connections).
#	cyrus does not flush cache until a disconnect so this is needed
#	before the back of mailboxes.db.
# cyrcreatembox(uid, machine, mailbox) - create a mailbox.
# cyrsubmbox(uid, machine, mailbox) - subscribe to a mailbox.
# cyrdeletembox(uid, machine, mailbox) - delete a mailbox.
#	note that deleting a users inbox will delete all subfolders.
#	this trick is used when deleting all of a user's folders.
#	Also note that the name inbox is a magic name.
#	Missing mailbox should not be an error on user delete.
# cyrquotaget(uid, machine) - get users quota info.
#	returns a list: (current storage, maximum storage).
# cyrquotaset(uid, machine, value) - set the users quota limit.
#	note quota cannot be removed through this API,
#	although the value of "NONE" is recognized and the
#	quota attribute will be removed from LDAP.
#	We rely on msgd to remove the actual quota root
#	for the user since that must occur on the physical machine
#	the user resides on.

use msgimap;

# variables local to this module.

# our connection cache, cyrclient is a hash:
# key is the machine, its value is the object pointer.
%cyrclient = ();

# does nothing at current
sub
cyrinit
{
	# msgimap_debug(1); # for protocol logging if you want it.
	# initialize the cache
	%cyrclient = ();
	return 1;
}

sub
cyrdone
{
	# disconnect from each session
	foreach $machine (keys(%cyrclient)) {
		my $socket = $cyrclient{"$machine"};
		close($socket);
	}
	%cyrclient = ();
}

sub
cyrcreatembox
{
	my ($uid, $machine, $mailbox, $noadmin) = @_;

	my $socket = cyrnew("$machine");

	my $mbox = $mailbox;

	my $altnamespace = msgimap_altnamespace();

	if ($noadmin ne "NOADMIN") {
		$mbox = cyrmkmbox($uid, $mailbox);
	} elsif (!$altnamespace) {
		$mbox = cyrmkmbox($uid, $mailbox);
	}

	# check if mailbox already exists, no error
	my $found = 0;
	my @mailboxes = msgimap_listmailbox($socket, $mbox, "%");
	if ($mailboxes[0] < 0) {
		print STDERR msgi18nsmsg(7, 40, $mbox); # Failed to list mailbox %1
		exit(1);
	}
	foreach my $item (@mailboxes) {
		if ($item eq $mbox) {
			$found = 1;
		}
	}
	if ($found) {
		return;
	}

	if (msgimap_createmailbox($socket, $mbox) < 0) {
		print STDERR msgi18nsmsg(2, 40, $mbox); # Failed to create mailbox %1
		exit(1);
	}
}

sub
cyrsubmbox
{
	my ($uid, $machine, $mailbox) = @_;

	my $socket = cyrnew("$machine");

	my $mbox = $mailbox;

	my $altnamespace = msgimap_altnamespace();

	if (!$altnamespace) {
		$mbox = cyrmkmbox($uid, $mailbox);
	}

	# check if mailbox exists, error if not

	msgimap_subscribemailbox($socket, $mbox);
}

sub
cyrdeletembox
{
	my ($uid, $machine, $mailbox) = @_;

	my $mbox = cyrmkmbox($uid, $mailbox);

	my $socket = cyrnew("$machine");

	# check if mailbox doesn't exist, no error
	my $found = 0;
	my @mailboxes = msgimap_listmailbox($socket, $mbox, "%");
	if ($mailboxes[0] < 0) {
		print STDERR msgi18nsmsg(7, 40, $mbox); # Failed to list mailbox %1
		exit(1);
	}
	foreach my $item (@mailboxes) {
		if ($item eq $mbox) {
			$found = 1;
		}
	}
	if (!$found) {
		return;
	}

        # set ACL so that we have delete permission
        if (msgimap_setaclmailbox($socket, $mbox, "$msgadmin", 'lca') < 0) {
		print STDERR msgi18nsmsg(4, 40, $mbox); # Failed to delete mailbox %1
		exit(1);
	}
 
        # delete the mbox
        if (msgimap_deletemailbox($socket, $mbox) < 0) {
		print STDERR msgi18nsmsg(4, 40, $mbox); # Failed to delete mailbox %1
		exit(1);
	}
}

sub
cyrquotaget
{
	my ($uid, $machine, $noadmin) = @_;

	my $socket = cyrnew("$machine");

	my $mbox = "INBOX";

	if ($noadmin ne "NOADMIN") {
		$mbox = cyrmkmbox($uid, "inbox");
	}

	my @quota = msgimap_getquota($socket, $mbox);
	if ($quota[0] < 0) {
		print STDERR msgi18nsmsg(6, 40, $mbox); # Failed to get quota on mailbox %1
		exit(1);
	}
	return(@quota);
}

sub
cyrquotaset
{
	my ($uid, $machine, $value) = @_;

	my $socket = cyrnew("$machine");

	my $mbox = cyrmkmbox($uid, "inbox");

	if (msgimap_setquota($socket, $mbox, $value) != 0) {
		print STDERR msgi18nsmsg(3, 40, $mbox); # Failed to set quota on mailbox %1
		exit(1);
	}
}

# our internal set of support routines, cannot have my sub in perl yet.

# make a new connection (or extract one from the cache)
# will connect and login
# usage: cyrnew(machine)
sub
cyrnew
{
	my ($machine) = @_;

	if (exists($cyrclient{"$machine"})) {
		return($cyrclient{"$machine"});
	}

	my $socket = msgimap_connect("$machine");
	if ($socket eq -1) {
		die msgi18nsmsg(1, 40);
	}
	my $ret = msgimap_login($socket, $msgadmin, $msgadminpw);
	if ($ret != 0) {
		print STDERR msgi18nsmsg(5, 40, $msgadmin); # Authentication failed for %1
		exit(1);
	}

	$cyrclient{"$machine"} = $socket;
	return($socket);
}

sub
cyrmkmbox
{
        my ($uid, $mailbox) = @_;
        my $sep  = msgimap_sep();

        if ($mailbox =~ /INBOX/i) {
                my $mbox = "user"."$sep"."$uid";
        } else {
                my $mbox = "user"."$sep"."$uid"."$sep"."$mailbox";
        }
}

return 1;	# required for perl modules.
