<?php
/*
 * Copyright (C) 2001-2002 Caldera International, Inc.  All Rights Reserved.
 */

/******************************************************************************
 * Class I18N
 *
 * Encapsulates functions used to which localized message files
 * to use in the GUI and to fetch appropriate messages. There would normally
 * be a single global non-persistent instance of this object.
 ******************************************************************************
 */
class I18N {
	var $lang;	// lang value with which the GUI will operate
	var $langs;	// array of langs available to the GUI
	var $messages;	// array containing the localized messages
	var $msgfile;	// the file where the localized messages are stored
	var $langbase = 'lang';

	/**********************************************************************
	 * Constructor I18N 
	 *
	 * Initialize the fields $lang and $msgfile. Uses the existing lang
	 * (global $session_lang) if it exists, otherwise it runs _initLang()
	 * method to determin the lang.
	 **********************************************************************
	 */
	function I18N () {
		global $session_lang;

		if ($session_lang == '') {
			$this->lang = $this->_initLang();
		} else {
			$this->lang = $session_lang;
		}
		$this->msgfile = $this->getMsgPath() . '/messages.dat';
		$this->loadIntlMsgs();
	}

	/**********************************************************************
	 * Public Function getLang()
	 *
	 * Accessor method for field $lang
	 **********************************************************************
	 */
	function getLang() {
		return $this->lang;
	}

	/**********************************************************************
	 * Public Function getLangPath()
	 *
	 * Accessor method for psedo-field $langPath. $langPath is the
	 * pathname used to access the top level language specific directory
	 * which contains help, messages, and images
	 **********************************************************************
	 */
	function getLangPath() {
		return $this->langbase . '/' . $this->lang;
	}

	/**********************************************************************
	 * Public Function getMsgPath()
	 *
	 * Accessor method for psedo-field $msgPath. $msgPath is the
	 * directory path which will contain the message catalog.
	 **********************************************************************
	 */
	function getMsgPath() {
		return $this->getLangPath() . '/messages';
	}

	/**********************************************************************
	 * Public Function getUnixLang() 
	 *
	 * Accessor method for psedo-field $unixLang. $unixLang is the
	 * formatted lang value that will be understood by msgcatopen
	 * on UNIX and Linux systems. It is used when the lang value needs
	 * to be passed the backend commandline via ENV variable
	 **********************************************************************
	 */
	function getUnixLang() {
		list($lower, $upper) = split('-', $this->lang, 2);
		$retval = $lower;
		if ($upper != '') {
			$retval.= '_' . strtoupper($upper);
		}
		return $retval;
	}

	/**********************************************************************
	 * Public Function loadIntlMsgs()
	 *
	 * This function assumes that $msgfile field has already been set
	 * (by the constructor). It parses that file in to an associative
	 * array with the msg id being the key and the msg itself as the
	 * value. The format of the message file must be:
	 * 	key:value
	 * with # being the comment character and \ at the end of a line
	 * escaping a newline which should be embedded in the message
	 **********************************************************************
	 */
	function loadIntlMsgs() {

		$lines = file($this->msgfile);
		
		$lines = array_reverse($lines);
		if ($lines != '') while ($lines) {
			// strip leading spaces
			$line = trim(array_pop($lines));

			// skip blank lines
			if ($line == '') {
				continue;
			}

			// skip comment lines
			$pos = strpos($line, '#');
			if ($pos !== false && $pos == 0) {
				continue;
			}

			list($id, $msg) = split(':', $line, 2);

			while (true) {
				if ($msg == '') break;

				if (substr($msg, -1) == '\\') {
					$this->messages[$id] .= substr($msg, 0, -1) . "\n";
					$msg = rtrim(array_pop($lines));
					continue;
				}

				$this->messages[$id] .= $msg;
				break;
			}

		}
	}

	/**********************************************************************
	 * Private Method _getLanguage()
	 *
	 * This method will fetch the value for MSG_LANGUAGE (a localized
	 * name for any given lang) from the specified message catalog.
	 * This is called by _initLanguages() 
	 *
	 * Parameters:
	 * 	o $file - string - message catalog file name
	 * Returns:
	 * 	Localized lang name or false if not found
	 **********************************************************************
	 */
	function _getLanguage($file) {
		$lines = file($file);
		
		$lines = array_reverse($lines);
		if ($lines != '') while ($lines) {
			// strip leading spaces
			$line = trim(array_pop($lines));

			// skip blank lines
			if ($line == '') {
				continue;
			}

			// skip comment lines
			$pos = strpos($line, '#');
			if ($pos !== false && $pos == 0) {
				continue;
			}

			list($id, $msg) = split(':', $line, 2);
			if ($id == 'MSG_LANGUAGE') {
				return $msg;
			}
		}

		return false;
	}


	/**********************************************************************
	 * Private Method _initLanguages()
	 *
	 * This method initializes the array of localized language names 
	 * corresponding to the langs available for use in the GUI. This
	 * can be used later to display a list to the user. The array is
	 * associative with the lang value as the key and the localized name
	 * as the value.
	 *
	 * NOTE: This method is not currently called by anything
	 **********************************************************************
	 */
	function _initLanguages() {
		foreach ($this->langs as $lang) {
			$language = 
				$this->_getLanguage($this->langbase . '/' .
					 $lang . '/messages.dat');

			if ($language !== false) {
				$this->languages[$lang] = $language;
			} else {
				$this->languages[$lang] = $lang;
			}
		}
	}

	/**********************************************************************
	 * function: _initLang()
	 *
	 * This function checks to see what the browser preferred languages are
	 * and finds the best language match among those available to the GUI.
	 * If no languages are available it is a fatal error. Otherwise, the
	 * function looks at each language specified by the browser and
	 * compares that to the list of installed languages. If a country
	 * specific language (en-us) is specified but isn't found a check
	 * is made for the  two-character lang (en) is made, if that is not
	 * found then the first country specific instance of that language 
	 * is used instead (e.g. en-uk). Similarly a country specific version
	 * of a language is used if the two-character lang is specified but
	 * not found. 
	 *
	 * If all the browser preferred language choices are exhausted then
	 * there is an attempt to default to 'en'. If 'en' is not available
	 * then the first encountered language is used.
	 *
	 * NOTE: Doesn't take into account a browser cookie lang value which
	 * 	 should override the browser env variable. Currently no such
	 * 	 cookie is read or written to the browser by the GUI. Perhaps
	 * 	 in the future
	 *
	 * Returns:
	 * 	The lang to use based on information from the browser and 
	 *	available langs. Plus, sets the following globals:
	 *	o langs array of available lang abbreviations (directory names)
	 **********************************************************************
	 */
	function _initLang() {
		global $HTTP_SERVER_VARS;

		// Parse the accept-lang string 
		$tmp = split(',', $HTTP_SERVER_VARS['HTTP_ACCEPT_LANGUAGE']);
		foreach ($tmp as $lang) {
			$accept[] = ereg_replace(';.*$', '', $lang);
		}

		// Get available languages
		unset($this->langs);
		$dh = opendir($this->langbase);
		while ($lang = readdir($dh)) {
			if (substr($lang,0,1) != '.') {
				$this->langs[] = $lang;
	
				// building the list of country specific backups
				// to internal version, last encountered wins
				if (strlen($lang) > 2) {
					$altlangs[substr($lang, 0, 2)] = $lang;
				}
			}
		}
		closedir($dh);

		// Eventually this should have a nicely formated page, too.
		if (count($this->langs) < 1) {
			print('<H1>ERROR: No language files available</H1>');
			exit();
		}

		// Find best match
		// Match first for exact lang or lang-country
		// Fallback to lang if lang-country not available
		$selectedLang = '';
		foreach ($accept as $lang) {
			$ilang = substr($lang, 0, 2);
			if (in_array($lang, $this->langs)) {
				$selectedLang = $lang;
				break;
			} elseif (in_array($ilang, $this->langs)) {
				// drop to international lang if necessary
				$selectedLang = $ilang;
				break;
			}
		}

		// Match now for an alternate lang-country of specific
		// lang or lang-country is not available.
		if ($selectedLang == '') foreach ($accept as $lang) {
			$ilang = substr($lang, 0, 2);
			if ($altlangs[$ilang] != '') {
				// above test should be 
				// array_key_exists($ilang, $altlangs)
				// when php version is updated

				// grap one of the available country specific
				// langs
				$selectedLang = $altlangs[$ilang];
				break;
			}
		}


		// Lang hasn't been found, default to en if available
		// or the first lang found if en isn't available
		// langs[0] must have a value or we would have errored
		// out above.
		if ($selectedLang == '') {
			if (in_array('en', $this->langs)) {
				$selectedLang = 'en';
			} else {
				$selectedLang = $this->langs[0];
			}
		}

		return $selectedLang;
	}

	/**********************************************************************
	 * Function addMessage()
	 *
	 * Adds a message to the message store. It will overwrite an existing
	 * message with the same message id.
	 *
	 * Paramters:
	 * o msgid - string - the id of the message to add
	 * o msg - string - the message to be added 
	 * Returns:
	 * nothing
	 **********************************************************************
	 */
	function addMessage($msgid, $msg) {
		$this->messages[$msgid] = $msg;
	}
	
}

/******************************************************************************
 * Function getIntlMsg()
 *
 * This takes a message id and returns an string from the 
 * correct message catalog. The function also takes a variable
 * number of additional arguments used to replace markers embedded in
 * the message string. Markers are %1, %2, etc.... All instances of a
 * marker (say %1) are replaces by the corresponding argument in the
 * argument array (arg(1)).
 *
 * NOTE: This has been externalized to reduce overhead since this function
 * is called a lot.
 *
 * Parameters:
 * 	o variable number of string parameters, first if the message id then a
 * 	  sequence of parameters to the message.
 ******************************************************************************
 */
function getIntlMsg() {
	global $INTL;

	// Initialize if first use
	if (! isset($INTL)) {
		$INTL = new I18N();
	}
	
	$numArgs = func_num_args();

	// must have at least one argument - the message id
	if ($numArgs < 1) {
		// DEBUG - needs an debug error case
		return '';
	}

	// get the message id - first argument
	$message = $INTL->messages[func_get_arg(0)];

	if ($message === '') {
		// DEBUG - needs an debug error case
		;
	}

	// process the remaining arguments
	for ($i = 1; $i < $numArgs; $i++) {
		// args and arg values need to be handled as strings
		$arg = "$i";
		$val = func_get_arg($i);
		$val = stripslashes($val);
		$tmp = ereg_replace("%$arg", "$val", $message);
	
		if ($tmp === $message) {
			// DEBUG - needs a debug error case for 
			// too many args or arg not found %$arg
			$message = $tmp;
		} else {
			$message = $tmp;
		}
	}

	if (ereg("%[0-9]+", $message)) {
		// DEBUG - needs a debug error case for 
		// not enough args
		;
	}

	return $message;
}

function getIntlMsgJS() {
	$args = func_get_args();
	$message = call_user_func_array('getIntlMsg', $args);
	$message = preg_replace("/'/", "\\'", $message);

	return $message;
}

?>
